import json
from urllib.parse import parse_qs

from django_requests_logger.models import RequestLog
from django_requests_logger.signals import response_is_not_ok, response_is_ok


def logger(response, data_masking=None, only_errors=False, *args, **kwargs):
    if only_errors and response.ok:
        return

    if "Content-Type" in response.headers and "application/json" in response.headers["Content-Type"]:
        content = response.json()
    else:
        content = response.text

    data = {
        "method": response.request.method,
        "url": response.request.url,
        "params": "",
        "headers": dict(response.request.headers),
        "response_content": content,
        "response_status_code": response.status_code,
        "response_headers": dict(response.headers),
    }
    if response.request.body:
        try:
            body = json.loads(response.request.body.decode())
        except Exception:
            try:
                body = parse_qs(response.request.body)
            except Exception:
                body = response.request.body

        if data_masking and isinstance(body, dict):
            body = masking(body, data_masking)

        data["body"] = body

    obj = RequestLog.objects.create(**data)
    if response.ok:
        response_is_ok.send(sender=RequestLog, instance=obj)
    else:
        response_is_not_ok.send(sender=RequestLog, instance=obj)


def masking(body, data_masking):
    return {k: "*****" if any((d in k for d in data_masking)) else v for k, v in body.items()}



# Generated by Django 2.0.8 on 2018-12-31 20:21

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='RequestLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('method', models.CharField(help_text='request method', max_length=10, verbose_name='method')),
                ('url', models.TextField(help_text='request url', verbose_name='url')),
                ('params', models.TextField(help_text='request params', verbose_name='parameters')),
                ('body', models.TextField(help_text='request body', verbose_name='body')),
                ('headers', models.TextField(help_text='request headers', verbose_name='headers')),
                ('response_content', models.TextField(help_text='response content', verbose_name='content')),
                ('response_headers', models.TextField(help_text='response headers', verbose_name='headers')),
                ('response_status_code', models.CharField(help_text='response status code', max_length=10, verbose_name='status code')),
                ('last_modified', models.DateTimeField(auto_now=True, help_text='last modification date', verbose_name='last modified')),
                ('created', models.DateTimeField(auto_now_add=True, help_text='creation date', verbose_name='created')),
            ],
            options={
                'verbose_name': 'Request Log',
                'verbose_name_plural': 'Request Logs',
                'db_table': 'request_log',
            },
        ),
    ]



from django.db import models
from django.utils.translation import ugettext_lazy as _


class RequestLog(models.Model):
    method = models.CharField(_('method'), max_length=10, help_text=_('request method'))
    url = models.TextField(_('url'), help_text=_('request url'))
    params = models.TextField(_('parameters'), help_text=_('request params'))
    body = models.TextField(_('body'), help_text=_('request body'))
    headers = models.TextField(_('headers'), help_text=_('request headers'))

    response_content = models.TextField(_('content'), help_text=_('response content'))
    response_headers = models.TextField(_('headers'), help_text=_('response headers'))
    response_status_code = models.CharField(_('status code'), max_length=10, help_text=_('response status code'))

    last_modified = models.DateTimeField(_('last modified'), auto_now=True, help_text=_('last modification date'))
    created = models.DateTimeField(_('created'), auto_now_add=True, help_text=_('creation date'))

    class Meta:
        db_table = 'request_log'
        verbose_name = 'Request Log'
        verbose_name_plural = 'Request Logs'



from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from django_requests_logger.models import RequestLog


class RequestLogAdmin(admin.ModelAdmin):
    list_display = (
        'url', 'method', 'response_status_code', 'created'
    )
    list_filter = ('method', 'response_status_code')
    search_fields = ['url']
    fieldsets = (
        (None, {
            'fields': (
                'url', 'method', 'response_status_code', 'created')
        }),
        (_('Request'), {
            'classes': ('collapse',),
            'fields': (
                'params', 'body', 'headers')
        }),
        (_('Response'), {
            'classes': ('collapse',),
            'fields': (
                'response_content', 'response_headers')
        })
    )
    readonly_fields = list(d for t in fieldsets for d in t[1]['fields'])


admin.site.register(RequestLog, RequestLogAdmin)



import os

from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="django-requests-logger",
    version="0.1.5",
    include_package_data=True,
    license="MIT",
    description="A django integration for requests.",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    url="https://github.com/GearPlug/django-requests-logger",
    author="Miguel Ferrer",
    author_email="ingferrermiguel@gmail.com",
    packages=["django_requests_logger"],
    install_requires=[
        "django",
        "requests",
    ],
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    zip_safe=False,
)
